#[non_exhaustive]pub enum SyscallFlagError<F: SyscallFlag> {
#[non_exhaustive] NotSupported {
flag: F,
set: bool,
},
}Expand description
Identifies errors when configuring a syscall flag.
Generic over the flag type to support different syscall flag categories.
Each flag type implements SyscallFlag.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
#[non_exhaustive]NotSupported
The requested flag configuration is not supported by the running kernel.