SyscallFlag

Trait SyscallFlag 

pub trait SyscallFlag:
    Copy
    + Debug
    + Sealed { }
Marker trait for syscall flag types used in compatibility checks.

This is the syscall-flag equivalent of the Access trait, but without the BitFlag requirement since syscall flags are set individually rather than as sets.

This trait is sealed and cannot be implemented outside of this crate.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors